/* Copyright (c) 2014 Nokia Solutions and Networks
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _FZM_CPLDSPI_H_
#define _FZM_CPLDSPI_H_

#ifdef __KERNEL__
typedef enum {
   FZM_CPLDSPI_CPLD = 1, /* do not change this enum list */
   FZM_CPLDSPI_FPGA_CONFIG,
} fzm_cpldspi_target_dev_t;

/* optional locking routines, required for shared registers */
void fzm_cpldspi_lock(fzm_cpldspi_target_dev_t dev);
int fzm_cpldspi_lock_interruptible(fzm_cpldspi_target_dev_t dev);
int fzm_cpldspi_trylock(fzm_cpldspi_target_dev_t dev);
void fzm_cpldspi_unlock(fzm_cpldspi_target_dev_t dev);

/* read and write routines for drivers with exclusive access to registers */
int fzm_cpldspi_read(fzm_cpldspi_target_dev_t dev,
                     unsigned addr, void *buf, size_t len);

int fzm_cpldspi_write(fzm_cpldspi_target_dev_t dev,
                      unsigned addr, const void *buf, size_t len);

/* single read/write for shared registers */
int fzm_cpldspi_read_locked(fzm_cpldspi_target_dev_t dev,
                            unsigned addr, void *buf, size_t len);

int fzm_cpldspi_write_locked(fzm_cpldspi_target_dev_t dev,
                             unsigned addr, const void *buf, size_t len);

#endif

#endif /* _FZM_CPLDSPI_H_ */
