/*****************************************************************************
 * Project Name: FSMr3
 * Module Name:  message dispatcher
 * File Name:    se_cfg_mod.c
 *
 * Author:        Junshu Zhang
 * Creation Date: 03 Aug 2010
 * Last modified: 18 Aug 2010
 * ---------------------------------------------------------------------------
 * Description: Kernel driver to encapsulate FPGA (and PLD) access
 *              includes new user-mode interface
 *****************************************************************************
 * Copyright (c) Nokia 2010, all rights reserved
 * Licensed material - property of Nokia
 *****************************************************************************
 * Revision history (latest on top)
 *
 * Date        | Who | Description
 * ------------+-----+--------------------------------------------------------
 * 19-Sep-2011 | JZh | added Abis support for GSM
 * 10-Oct-2010 | JZh | added error code
 * 02-Sep-2010 | DSi | Increased SECFG_MSG_LONG_MAXLEN to 1024
 * 20-Aug-2010 | JZh | generic support for ATM and IPSec SEs
 * 03-Aug-2010 | JZh | init version to support ATM config
 * 25-Jun-2010 | GFr | init version echo reply
 *****************************************************************************/

/*************************************************************************
 * Header file to specify the commands, attributes and policies used in 
 * the Generic Netlink messages between config drivers and kernel module
 *************************************************************************/
#ifndef seCfgDrv_If_h
#define seCfgDrv_If_h

#ifdef __cplusplus
extern "C" {
#endif

/* struct nla_policy. 
 * Header file location is different for kernel and user space
 */
#ifdef __KERNEL__
#include <net/netlink.h>
#else
#include <netlink/attr.h>
#endif

#define SECFG_GENL_FAMILY_NAME      "SE Config"
#define SECFG_GENL_PROTOCOL_VERSION (1)
#define PENDING_MSG_MAX             (16) /* num of pending msgs */
/*
#define ATMSE_REC_POW               (9)
#define IPSE_REC_POW                (7)
*/


/* Important:
 * keep the XXX_REQUEST and XXX_REPLY cmd in this order and in pair
 * This mean: XXX_REPLY = XXX_REQUEST + 1
 */
enum eSeCfgDrvGenlCommands {
    C_SECFG_UNSPEC,        /* keep this first, genl doesn't like index 0 */

    /* IPSec SE configuration*/ 
    C_IP_FIRST_ENTRY,                    /* first entry for IP SE */
    C_IP_CFG_REQUEST = C_IP_FIRST_ENTRY, /* use A_SECFG_MSG_LONG */
    C_IP_CFG_REPLY,                      /* use A_SECFG_MSG_RET */
    C_IP_PERF_DATA_REQUEST,
    C_IP_PERF_DATA_REPLY,
    C_IP_DEBUG_REQUEST,
    C_IP_DEBUG_REPLY,
    C_SE_TO_HOST_PID,
    C_IPCS_CFG_REQUEST,
    C_IP_LAST_ENTRY =  C_SE_TO_HOST_PID,  /* last entry for IP SE */
    /* ATM SE configuration */
    C_ATM_FIRST_ENTRY,                      /* first entry for ATM SE */
    C_ATM_CFG_REQUEST = C_ATM_FIRST_ENTRY,  /* use A_SECFG_MSG_LONG */
    C_ATM_CFG_REPLY,                        /* use A_SECFG_MSG_RET */
    C_ATM_PERF_DATA_REQUEST,
    C_ATM_PERF_DATA_REPLY,
    C_ATM_DEBUG_REQUEST,
    C_ATM_DEBUG_REPLY,
    C_ATM_LAST_ENTRY = C_ATM_DEBUG_REPLY,   /* last entry for ATM SE */

    /* ABIS SE configuration */
    C_ABIS_FIRST_ENTRY,                       /* first entry for ABIS SE */
    C_ABIS_CFG_REQUEST = C_ABIS_FIRST_ENTRY,  /* use A_SECFG_MSG_LONG */
    C_ABIS_CFG_REPLY,                         /* use A_SECFG_MSG_RET */
    C_ABIS_PERF_DATA_REQUEST,
    C_ABIS_PERF_DATA_REPLY,
    C_ABIS_DEBUG_REQUEST,
    C_ABIS_DEBUG_REPLY,
    C_ABIS_LAST_ENTRY = C_ABIS_DEBUG_REPLY,   /* last entry for ABIS SE */

    /* last one which sends to se */
    C_SE_LAST_ENTRY = C_ABIS_LAST_ENTRY,      

    /* test */
    C_SECFG_TEST,
    C_SECFG_ECHO_REQUEST,
    C_SECFG_ECHO_REPLY,    
    C_SECFG_ARRAY_LIMIT    /* keep this last */
};
#define SECFG_CMD_MAX (C_SECFG_ARRAY_LIMIT)

enum eSeCfgDrvGenlAttrs {
    A_SECFG_UNSPEC,        /* keep this first, netlink doesn't like index 0  */
    A_SECFG_ECHO_STRING,   /* string to pass in echo */
    A_SECFG_MSG_SHORT,
    A_SECFG_MSG_LONG,
    A_SECFG_MSG_RET,
    A_SECFG_MSG_RET_U32,
    A_SECFG_ARRAY_LIMIT    /* keep this last */
};
#define SECFG_ATTR_MAX (A_SECFG_ARRAY_LIMIT)

#define SECFG_MAX_STRING_LEN      (64)
#define SECFG_MSG_SHORT_MAXLEN    (96)
#define SECFG_MSG_LONG_MAXLEN     (1024)  // DSi - IP DA can send upto 1024 byted of data, 
                                          //       so updated to accomodate this
                                          //       This NEEDS to be reworked.

enum target_se {
    TARGET_SE_ATM,
    TARGET_SE_IP,
    TARGET_SE_ABIS,
    TARGET_SE_NONE
};


#define USE_PACKET_BUF(x)          (x)

enum errcode {
    EM_SUCCESS,

    EM_RX_NL_FIRST,
    EM_RX_NL_EMPTY,
    EM_RX_NL_NOATTR,
    EM_RX_NL_NODATA,
    EM_RX_NL_DATAZERO,
    EM_RX_NL_LAST,

    EM_MPOOL_FIRST,
    EM_MPOOL_TX_PENDING,
    EM_MPOOL_RX_PENDING,
    EM_MPOOL_LAST,

    EM_TX_SE_FIRST,
    EM_TX_SE_NOWQE,
    EM_TX_SE_NOBUF,
    EM_TX_SE_UNKNOWNSE,
    EM_TX_SE_LAST,

    EM_RX_SE_FIRST,
    EM_RX_SE_TIMEOUT,
    EM_RX_SE_WQENULL,
    EM_RX_SE_RCV_ERR,
    EM_RX_SE_CPBUF_ALLOC,
    EM_RX_SE_LAST,

    EM_TX_NL_FIRST,
    EM_TX_NL_NOSKB,
    EM_TX_NL_NOMAGIC,
    EM_TX_NL_MSGHEAD,
    EM_TX_NL_NLA_PUT,
    EM_TX_NL_GENLMSG_END,
    EM_TX_NL_GENLMSG_REPLY,
    EM_TX_NL_LAST
};


#ifdef __KERNEL__
typedef struct{
    /* netlink parameters */
    unsigned  pid;       // who sent the config msg
    unsigned  seq_num;   // genl sequence number
    unsigned  msg_len;   // the netlink msg length
    unsigned  cmd_req;   // request command: always = cmd_idx
    unsigned  cmd_rpy;   // reply command
    /* se config parameters */
    unsigned  seq_cfg;   // se cfg msg sequence number
    unsigned  status;    // none zero=entry used; zero=entry free.
    unsigned  cmd_type;  // u32, binary, string, etc.
    unsigned  se;        // target SE: ATM or IPSec
    /* POW parameters */
    unsigned  pow_grp;   // target SE receiving POW group
    unsigned  tag;       // target SE POW tag
    unsigned  tag_type;  // target SE POW tag type
    unsigned  use_pkt_buf; // none zero use pkt buf; zero use WQE
    unsigned  pow_qos; 
    /* return msg */
    void *    p_rtmsg;
    unsigned  len_rtmsg;
    unsigned  rx_ret;

    /* counters */
    unsigned  ct_nl_rx;      // user->kernel msg received
    unsigned  ct_nl_rx_data; // user->kernel msg received, but bad
    unsigned  ct_pool_err;   // msg pool conflict
    unsigned  ct_se_tx_err;  // kernel->se sent failed
    unsigned  ct_se_tx;      // kernel->se sent
    unsigned  ct_se_rx;      // se->kernel received
    unsigned  ct_se_rx_timeout; // se->kernel received, but bad
    unsigned  ct_nl_tx_err; // kernel->user prepare send failed
    unsigned  ct_nl_tx;      // kernel->user sent
} pending_msg_t;

//int seCfgDrv_init(void);
//void seCfgDrv_exit(void);

typedef struct{
    /**
     * use the netlink cmd as pending msg pool idx
     * to avoid pool access conflict under SMP
     */
    pending_msg_t entry[C_SECFG_ARRAY_LIMIT];
//    pending_msg_t entry[PENDING_MSG_MAX];
    
} pending_msg_pool_t;

/* family definition */
/*
struct genl_family seCfg_genl_family = {
    .id      = GENL_ID_GENERATE,
    .hdrsize = 0,
    .name    = SECFG_GENL_FAMILY_NAME,
    .version = SECFG_GENL_PROTOCOL_VERSION,
    .maxattr = SECFG_ATTR_MAX,
};
*/
#endif


#ifndef __KERNEL__
static struct nla_policy seCfg_genl_policy[A_SECFG_ARRAY_LIMIT] = {
    [A_SECFG_ECHO_STRING] = {.type = NLA_STRING,
#ifdef __KERNEL__
                             .len  = SECFG_MAX_STRING_LEN,
#else
                             .maxlen = SECFG_MAX_STRING_LEN,
#endif
    },
    /* unfortunately kernel and user-space disagree on how to write attr policies */
    [A_SECFG_MSG_SHORT] = {
#ifdef __KERNEL__
                          .type   = NLA_BINARY,
                          .len    = SECFG_MSG_SHORT_MAXLEN,
#else
                          .type   = NLA_UNSPEC,
                          .minlen = 0,
                          .maxlen = SECFG_MSG_SHORT_MAXLEN,
#endif
    },
    [A_SECFG_MSG_LONG] = {
#ifdef __KERNEL__
                          .type   = NLA_BINARY,
                          .len    = SECFG_MSG_LONG_MAXLEN,
#else
                          .type   = NLA_UNSPEC,
                          .minlen = 0,
                          .maxlen = SECFG_MSG_LONG_MAXLEN,
#endif
    },
    [A_SECFG_MSG_RET] = {
#ifdef __KERNEL__
                          .type   = NLA_BINARY,
                          .len    = SECFG_MSG_LONG_MAXLEN,
#else
                          .type   = NLA_UNSPEC,
                          .minlen = 0,
                          .maxlen = SECFG_MSG_LONG_MAXLEN,
#endif
    },
    [A_SECFG_MSG_RET_U32] = {.type = NLA_U32,
                         /* pre-defined length */
    },

};
#endif

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif

/* EOF */
