/**
*******************************************************************************
* @file                  ARM_DSP_comm_shared.h
* @version               Draft
* @date                  20-Mar-2013
* @author                LSP BLS Team
*
* @brief                 Shared definitions
* @module                ARM_DSP_comm_shared
* @owner                 LSP BLS Team
*
* Copyright 2013-2014 Nokia Solutions and Networks. All rights reserved.
*******************************************************************************/

#ifndef ARMDSP_COMM_H
#define ARMDSP_COMM_H


/* Linux reserved HW queue IDs are as follows */
/* 
Queues: 652-671 (Low priority interrupt queues)
Queues: 8704-8735 (ARM interrupt controller queue pend queues)
Queues: 4000-4063 (General purpose)
Queues: 640-648 (PA TX queues)
Queues: 800-811 (PktDma Transmit Queues)
Queues: 812-813 (PktDma Transmit Queues FZM interim solution)
*/
/* All other queue IDs not reserved in the range 0-9023 potentially available for all cores */
/* HW queue ID split 50/50 of general purpose queues */
#define ADCOMM_QID_GP_DSP_FIRST 9024
#define ADCOMM_QID_GP_DSP_LAST  12703
#define ADCOMM_QID_GP_ARM_FIRST ((ADCOMM_QID_GP_DSP_LAST) + 1)
#define ADCOMM_QID_GP_ARM_LAST  16383

/* RX Flow split */
#define ADCOMM_RX_FLOW_ID_ARM_FIRST 0
#define ADCOMM_RX_FLOW_ID_ARM_LAST  31
#define ADCOMM_RX_FLOW_ID_DSP_FIRST ((ADCOMM_RX_FLOW_ID_ARM_LAST) + 1)
#define ADCOMM_RX_FLOW_ID_DSP_LAST  ((ADCOMM_RX_FLOW_ID_DSP_FIRST) + 31)

/* Linux reserved RX Flow ID's are 0-11, 22-23, and 31 */
/* RX Flow ID for DSPs to ARM communication */
#define ADCOMM_RX_FLOW_ID_ARM_RX  ((ADCOMM_RX_FLOW_ID_ARM_FIRST) + 12)

/* RX Flow IDs for ARM to DSPs communication */
#define ADCOMM_RX_FLOW_ID_DSP0_RX  ((ADCOMM_RX_FLOW_ID_DSP_FIRST) + 0)
#define ADCOMM_RX_FLOW_ID_DSP1_RX  ((ADCOMM_RX_FLOW_ID_DSP_FIRST) + 1)
#define ADCOMM_RX_FLOW_ID_DSP2_RX  ((ADCOMM_RX_FLOW_ID_DSP_FIRST) + 2)
#define ADCOMM_RX_FLOW_ID_DSP3_RX  ((ADCOMM_RX_FLOW_ID_DSP_FIRST) + 3)
#define ADCOMM_RX_FLOW_ID_DSP4_RX  ((ADCOMM_RX_FLOW_ID_DSP_FIRST) + 4)
#define ADCOMM_RX_FLOW_ID_DSP5_RX  ((ADCOMM_RX_FLOW_ID_DSP_FIRST) + 5)
#define ADCOMM_RX_FLOW_ID_DSP6_RX  ((ADCOMM_RX_FLOW_ID_DSP_FIRST) + 6)
#define ADCOMM_RX_FLOW_ID_DSP7_RX  ((ADCOMM_RX_FLOW_ID_DSP_FIRST) + 7)

/* Internal Fast Linking RAM split 32K total indexes */
/* ARM FLR is 8K */
#define ADCOMM_ARM_FLR_FIRST_IDX  0x4000
#define ADCOMM_ARM_FLR_LAST_IDX   0x5fff

/* DSP FLR is all other indecies */
#define ADCOMM_DSP_FLR_FIRST_IDX1 0x0
#define ADCOMM_DSP_FLR_LAST_IDX1  ((ADCOMM_ARM_FLR_FIRST_IDX)-1)
#define ADCOMM_DSP_FLR_FIRST_IDX2 ((ADCOMM_ARM_FLR_LAST_IDX)+1)
#define ADCOMM_DSP_FLR_LAST_IDX2  0x7fff

/* PDSP split */
#define ADCOMM_DSP_PDSP_FIRST     1
#define ADCOMM_DSP_PDSP_LAST      4
#define ADCOMM_ARM_PDSP_FIRST     5
#define ADCOMM_ARM_PDSP_LAST      8

/* Shared memory in first part of DSP DDR3A memory */
#define ADCOMM_SHARED_MEM_ADDR_36    (0x800080000u)
#define ADCOMM_SHARED_MEM_ADDR_32    (0x80080000u)

#define ADCOMM_DB_MAGIC_NUM   0xc0edbabe

typedef struct
{
    u32 magic_num;
    u32 dsp_nid[8];
    u32 arm_nid;
} ADCOMM_DB_T;

#endif /*ARMDSP_COMM_H*/
