// *****************************************************************************
// PROJECT NAME     :   FZM
// Component name   :   SCTPLPBK
// Filename         :   
// Author           :   Binny/Abhilash
// Date created     :   03.09.2013
// Date changed     :   -
// -----------------------------------------------------------------------------
// Description      :  Implements SCTP Loopback Functionality 
// -----------------------------------------------------------------------------
// Revision history :
// date       | author         | comments
// ---------- + -------------- +----------------------------------------------------
// 19.02.2013 | Binny Jeshan   | Initial version
//            |                |
// *****************************************************************************
// Copyright (c) Nokia 2013
// All rights reserved.
// Licensed material - property of Nokia
// *****************************************************************************

#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <linux/netfilter_ipv6.h>
#include <linux/skbuff.h>
#include <linux/module.h>
#include <linux/udp.h>
#include <net/ip.h>
#include <net/route.h>
#include <net/ip6_route.h>
#include <linux/in.h>
#include <linux/inet.h>

#define IWF_IPV4_VERSION  4
#define IWF_IPV6_VERSION  6
#define IPIPV6ADDRESSLENGTH 16
/* globals */
struct   proc_dir_entry *fzm_cplane_d  = NULL;
struct   proc_dir_entry *fzm_cplane_ip_help_f = NULL;
static int cplane_ip_help_proc_read(struct seq_file *m, void *v);

static int cplane_ip_help_open(struct inode *inode, struct file *file)
{
    return single_open(file,cplane_ip_help_proc_read, NULL);
}
static const struct file_operations cplane_proc_ip_help_fops = {
    .owner = THIS_MODULE,
    .open  = cplane_ip_help_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release
};


static struct nf_hook_ops  netfilter_uplink_ops;
static struct nf_hook_ops  netfilter_uplink_ipv6_ops;

static  char saddr1[] = "11.0.0.1";
static  char daddr1[] = "30.30.30.10";

static  char saddr2[] = "15.0.0.1";
static  char daddr2[] = "40.40.40.10";


static  char saddr3[] = "16.0.0.1";
static  char daddr3[] = "50.50.50.10";

static  char saddr4[] = "17.0.0.1";
static  char daddr4[] = "60.60.60.10";


void printPacket(char *mesg, unsigned char *p, int len)
{

    int newline=10;
    printk("         %s         ",mesg);
    printk("\n---------------------------------- \n");
    while(len--)
    {
        printk(" %.2X", *p);
        p++;
        newline--;
        if(0 == newline)
        {
            newline=10;
            printk("\n");
        }
    }
    printk("\n---------------------------------- \n");
    printk("\n\n");
}

unsigned int sctpLb_UpLinkHook(unsigned int hooknum,
        struct sk_buff *skb,
        const struct net_device *in,
        const struct net_device *out,
        int (*okfn)(struct sk_buff*))
{
    int iPktLen;   
    struct iphdr   *ip_header    =  NULL;
    struct ipv6hdr *ip6_header   =  NULL;
    struct rtable       *rt      =  NULL;
    struct rt6_info     *rtv6     =  NULL;
    struct net_device   *dev     =  NULL;

    /* if not NULL */ 
    if(skb)
    {
       //intk("\n hello ");
        dev          = skb->dev;
        ip_header  =  (struct iphdr *)skb->data;
        /* if not fragment packet & if its a udp packet */
        //if( (!ip_is_fragment(ip_header)) && (IPPROTO_UDP == ip_header->protocol) )

        if(ip_header != NULL)
        {
            if (ip_header->version == IWF_IPV4_VERSION)
            {
                if( (IPPROTO_SCTP == ip_header->protocol) )
                {
                    iPktLen = skb_is_nonlinear(skb) ? skb_headlen(skb) : skb->len;
                    if(iPktLen > 0)
                    {

                        /* fill the destination ip i.e. S-GW IP and source ip */

                        if(ip_header->daddr == (unsigned long)in_aton (saddr1))
                        {
                            ip_header->saddr = (unsigned long)in_aton (saddr1);
                            ip_header->daddr = (unsigned long)in_aton (daddr1);
                        }
                        else if(ip_header->daddr == (unsigned long)in_aton (saddr2))
                        {

                            ip_header->saddr = (unsigned long)in_aton (saddr2);
                            ip_header->daddr = (unsigned long)in_aton (daddr2);
                        }
                        else if(ip_header->daddr == (unsigned long)in_aton (saddr3))
                        {

                            ip_header->saddr = (unsigned long)in_aton (saddr3);
                            ip_header->daddr = (unsigned long)in_aton (daddr3);
                        }
                        else if(ip_header->daddr == (unsigned long)in_aton (saddr4))
                        {
                            ip_header->saddr = (unsigned long)in_aton (saddr4);
                            ip_header->daddr = (unsigned long)in_aton (daddr4);
                        }
                        else
                        {
                            printk("INVALID IP !..\n");
                        }

                        /*recompute ip header checksum*/
                        ip_send_check(ip_header);

                        /* do route lookup */
                        rt = ip_route_output(dev_net(dev), ip_header->daddr, ip_header->saddr,
                                (ip_header->tos), 0);

                        if (!IS_ERR(rt)) 
                        {
                            /*  fill in route details in skb */
                            skb_dst_set(skb, &rt->dst);
                            dst_output(skb);

                        }else{   
                            //printk("\n DROP no route \n"); 
                            dev_kfree_skb_any(skb); 
                            skb = NULL; 
                        }   
                        return NF_STOLEN;
                    }
                }
            }else if (ip_header->version == IWF_IPV6_VERSION)
            {
                ip6_header =  (struct ipv6hdr *)skb->data;

                if (ip6_header != NULL)
                {
                    if(IPPROTO_SCTP == ip6_header->nexthdr)
                    {
                        iPktLen = skb_is_nonlinear(skb) ? skb_headlen(skb) : skb->len;

                        if(iPktLen >0)
                        {

                            //operator1
                            if(ntohl(ip6_header->daddr.s6_addr32[0]) ==(0xfc800000))
                            {   

                                ip6_header->saddr.s6_addr32[0] = htonl(0xfc800000);
                                ip6_header->saddr.s6_addr32[1] = htonl(0x020fbbff);
                                ip6_header->saddr.s6_addr32[2] = htonl(0xfe845eb2);
                                ip6_header->saddr.s6_addr32[3] = htonl(0x00000006);

                                ip6_header->daddr.s6_addr32[0] = htonl(0xfc800000);
                                ip6_header->daddr.s6_addr32[1] = htonl(0x020fbbff);
                                ip6_header->daddr.s6_addr32[2] = htonl(0xfe845eb2);
                                ip6_header->daddr.s6_addr32[3] = htonl(0x00000002);

                            }

                            //operator2

                            else if( ntohl(ip6_header->daddr.s6_addr32[0]) ==(0xfc810000))
                            {

                                ip6_header->saddr.s6_addr32[0] = htonl(0xfc810000);
                                ip6_header->saddr.s6_addr32[1] = htonl(0x020fbbff);
                                ip6_header->saddr.s6_addr32[2] = htonl(0xfe845eb2);
                                ip6_header->saddr.s6_addr32[3] = htonl(0x00000006);

                                ip6_header->daddr.s6_addr32[0] = htonl(0xfc810000);
                                ip6_header->daddr.s6_addr32[1] = htonl(0x020fbbff);
                                ip6_header->daddr.s6_addr32[2] = htonl(0xfe845eb2);
                                ip6_header->daddr.s6_addr32[3] = htonl(0x00000002);
                            }

                            //operator3

                            else if(ntohl (ip6_header->daddr.s6_addr32[0]) ==(0xfc820000))
                            {

                                ip6_header->saddr.s6_addr32[0] = htonl(0xfc820000);
                                ip6_header->saddr.s6_addr32[1] = htonl(0x020fbbff);
                                ip6_header->saddr.s6_addr32[2] = htonl(0xfe845eb2);
                                ip6_header->saddr.s6_addr32[3] = htonl(0x00000006);

                                ip6_header->daddr.s6_addr32[0] = htonl(0xfc820000);
                                ip6_header->daddr.s6_addr32[1] = htonl(0x020fbbff);
                                ip6_header->daddr.s6_addr32[2] = htonl(0xfe845eb2);
                                ip6_header->daddr.s6_addr32[3] = htonl(0x00000002);
                            }

                            //operator 4

                            else if(ntohl (ip6_header->daddr.s6_addr32[0]) ==(0xfc830000))
                            {

                                ip6_header->saddr.s6_addr32[0] = htonl(0xfc830000);
                                ip6_header->saddr.s6_addr32[1] = htonl(0x020fbbff);
                                ip6_header->saddr.s6_addr32[2] = htonl(0xfe845eb2);
                                ip6_header->saddr.s6_addr32[3] = htonl(0x00000006);

                                ip6_header->daddr.s6_addr32[0] = htonl(0xfc830000);
                                ip6_header->daddr.s6_addr32[1] = htonl(0x020fbbff);
                                ip6_header->daddr.s6_addr32[2] = htonl(0xfe845eb2);
                                ip6_header->daddr.s6_addr32[3] = htonl(0x00000002);
                            }

                            else
                            {
                                printk("Invalid operator!..\n");
                            }
                            #if 0
                             printk(KERN_INFO "\nafter swap %s %d Dst Address: %08X %08X %08X %08X", __FUNCTION__, __LINE__,
                                    ip6_header->daddr.s6_addr32[0],
                                    ip6_header->daddr.s6_addr32[1],
                                    ip6_header->daddr.s6_addr32[2],
                                    ip6_header->daddr.s6_addr32[3]);

                             printk(KERN_INFO "\after swap n %s %d Source Address: %08X %08X %08X %08X", __FUNCTION__, __LINE__,
                                    ip6_header->saddr.s6_addr32[0],
                                    ip6_header->saddr.s6_addr32[1],
                                    ip6_header->saddr.s6_addr32[2],
                                    ip6_header->saddr.s6_addr32[3]);
                            #endif
                            /* route lookup */
                            rtv6 = rt6_lookup(dev_net(dev), &ip6_header->daddr, &ip6_header->saddr, 0, 0);

                            if (rtv6!= NULL)
                            {
                                /*  fill in route details in skb */
                                skb_dst_set(skb, &rtv6->dst);
                                dst_output(skb);

                            }else{
                                //printk("\n DROP no route \n"); 
                                dev_kfree_skb_any(skb); 
                                skb = NULL; 
                            } 

                            return NF_STOLEN;

                        }

                    }
                }

            }//IPv6 

        }

    }//skb
    return NF_ACCEPT;
}

int __init test_cplane_init(void)
{

    netfilter_uplink_ops.hook              =       sctpLb_UpLinkHook;
    netfilter_uplink_ops.pf                =       PF_INET;
    netfilter_uplink_ops.hooknum           =       NF_INET_PRE_ROUTING;
    netfilter_uplink_ops.priority          =       NF_IP_PRI_FIRST;
    nf_register_hook(&netfilter_uplink_ops);

    netfilter_uplink_ipv6_ops.hook              =       sctpLb_UpLinkHook;
    netfilter_uplink_ipv6_ops.pf                =       NFPROTO_IPV6;
    netfilter_uplink_ipv6_ops.hooknum           =       NF_INET_PRE_ROUTING;
    netfilter_uplink_ipv6_ops.priority          =       NF_IP6_PRI_FIRST;
    nf_register_hook(&netfilter_uplink_ipv6_ops);

    fzm_cplane_d = proc_mkdir("fzm_cplane_ip", NULL);
    if(NULL == fzm_cplane_d)
    {
        return -ENOMEM;
    }
    fzm_cplane_ip_help_f = proc_create("cplane_ko_ip_help", 0644 , fzm_cplane_d , &cplane_proc_ip_help_fops);
    if(NULL == fzm_cplane_ip_help_f)
    {
        remove_proc_entry("cplane_ko_ip_help", NULL);
        return -ENOMEM;
    }

    return 0;
}

static int cplane_ip_help_proc_read(struct seq_file *m, void *v)
{
    seq_printf(m,"=============================================================== \n");
    seq_printf(m,"            CPLANE STUB DEFAULT IPS \n");
    seq_printf(m,"--------------------------------------------------------------- \n");
    seq_printf(m,"     OPERATOR                     IPV4 IP  \n");
    seq_printf(m,"--------------------------------------------------------------- \n");
    seq_printf(m,"   saddr_operator_1            %s \n",saddr1);
    seq_printf(m,"   daddr_operator_1            %s \n\n",daddr1);
    seq_printf(m,"   saddr_operator_2            %s \n",saddr2);
    seq_printf(m,"   daddr_operator_2            %s \n\n",daddr2);
    seq_printf(m,"   saddr_operator_3            %s \n",saddr3);
    seq_printf(m,"   daddr_operator_3            %s \n\n",daddr3);
    seq_printf(m,"   saddr_operator_4            %s \n",saddr4);
    seq_printf(m,"   daddr_operator_4            %s \n\n",daddr4);
    seq_printf(m,"--------------------------------------------------------------- \n");
    seq_printf(m,"     OPERATOR                     IPV6 IP  \n");
    seq_printf(m,"--------------------------------------------------------------- \n");
    seq_printf(m,"   saddr_operator_1            fc80:0:20f:bbff:fe84:5eb2:0:6 \n");
    seq_printf(m,"   daddr_operator_1            fc80:0:20f:bbff:fe84:5eb2:0:2 \n\n");
    seq_printf(m,"   saddr_operator_2            fc81:0:20f:bbff:fe84:5eb2:0:6 \n");
    seq_printf(m,"   daddr_operator_2            fc81:0:20f:bbff:fe84:5eb2:0:2 \n\n");
    seq_printf(m,"   saddr_operator_3            fc82:0:20f:bbff:fe84:5eb2:0:6 \n");
    seq_printf(m,"   daddr_operator_3            fc82:0:20f:bbff:fe84:5eb2:0:2 \n\n");
    seq_printf(m,"   saddr_operator_4            fc83:0:20f:bbff:fe84:5eb2:0:6 \n");
    seq_printf(m,"   daddr_operator_4            fc83:0:20f:bbff:fe84:5eb2:0:2 \n\n");
    seq_printf(m,"===============================================================  \n");
    return 0;
}


void __exit test_cplane_exit(void)
{
    nf_unregister_hook(&netfilter_uplink_ops);
    nf_unregister_hook(&netfilter_uplink_ipv6_ops);

    if(NULL != fzm_cplane_d)
    {
        remove_proc_entry("cplane_ko_ip_help", fzm_cplane_d);
    }


}

/* ------------------------------------------------------------------------------ */
module_init(test_cplane_init);
module_exit(test_cplane_exit);

MODULE_AUTHOR("Nokia Solutions and Networks");
MODULE_DESCRIPTION("SCTP loopback");
MODULE_LICENSE("GPL");

