
#define LINUX

#include <linux/of.h>

#include <linux/time.h>

#include <linux/ip.h>
#include <linux/udp.h>

#include <linux/io.h>

#include <linux/dma-mapping.h>

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/netdevice.h>
#include <linux/platform_device.h>
#include <linux/keystone-dma.h>
#include <net/sock.h>
#include <net/netlink.h>


// TODO: remove this header file and replace with the real Syscom header
#include "SicadInfo.h"

static unsigned int ether_loop_back = 0;
static unsigned int arm_loop_back = 0;
static unsigned int dsp_id = 1;
static __be16 iperf_test_port_ne;
static unsigned int iperf_test_port = 5555;
static unsigned int ether_bridge = 0;

module_param(ether_loop_back, uint, 0);
module_param(arm_loop_back, uint, 0);
module_param(dsp_id, uint, 0);
module_param(iperf_test_port, uint, 0);
module_param(ether_bridge, uint, 0);

#include "msg_rtr_arm.h"

struct tester_global_data {
	struct net_device *dev;
	struct iphdr ip;
	struct udphdr udp;
	__be16 return_port;
	__be16 protocol;
    struct net_device *eth0;
    struct net_device *eth1;
};

static struct tester_global_data tester_global;

static int tester_sendpacket(struct sk_buff *skb)
{
  unsigned char dest[ETH_ALEN]
           = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };

    if (!(tester_global.dev->flags & IFF_UP))
        return -ENETDOWN;

    skb->protocol = tester_global.protocol;
    skb->dev = tester_global.dev;
	
    skb_pull(skb, sizeof(struct SMsgData));
    dev_hard_header(skb, tester_global.dev, ETH_P_IP, dest, tester_global.dev->dev_addr, tester_global.dev->addr_len);
    dev_queue_xmit(skb);

    return 0;

}

static unsigned int tester_net(unsigned int hooknum,
                                struct sk_buff *skb,
                                const struct net_device *in,
                                const struct net_device *out,
                                int (*okfn)(struct sk_buff *))
{
	__be32 tmpaddr;
	struct iphdr * ip = (struct iphdr *)(skb->data);
        struct udphdr *udp;
	struct SMsgData* syscomhdr;

        if (ip->protocol != IPPROTO_UDP)
                return NF_ACCEPT;

        udp = (struct udphdr *)((u8 *)ip + (ip->ihl << 2));

	if (udp->dest == iperf_test_port_ne)
	{
		tester_global.return_port = udp->source;	
	}
	else if (udp->dest != tester_global.return_port) {
		return NF_ACCEPT;
	}

    tester_global.protocol = skb->protocol;

    if (ether_bridge) 
    {
        if (skb->dev == tester_global.eth1)
        {
            tester_global.dev = tester_global.eth0;
        }
        else
        {
            tester_global.dev = tester_global.eth1;
        }
    }
    else
    {
        tester_global.dev = skb->dev;
        tmpaddr = ip->saddr;
        ip->saddr = ip->daddr;
        ip->daddr = tmpaddr;
    }
	

	syscomhdr = (struct SMsgData*)skb_push(skb, sizeof(*syscomhdr));
        syscomhdr->header.sender = dsp_id << 16;
        syscomhdr->header.target = dsp_id << 16;
        syscomhdr->header.msgSize = skb->len;
        syscomhdr->header.flags = skb->len;

	if (ether_loop_back)
	{
		tester_sendpacket(skb);
		return NF_STOLEN;
	}
	if (arm_loop_back)
	{
        	if (MsgRtrARMSend_loopback(skb) < 0)
		{
        		dev_kfree_skb_any(skb);
		}
	} else { 
        	if (MsgRtrARMSend(skb) < 0)
		{
        		dev_kfree_skb_any(skb);
		}
	}
	return NF_STOLEN;
}




static struct nf_hook_ops tester_net_ops[] = {
        {
                .hook           = tester_net,
                .owner          = THIS_MODULE,
                .pf             = NFPROTO_IPV4,
                .hooknum        = NF_INET_LOCAL_IN,
                .priority       = 4000,
        },
};


static int tester_setup_nethooks (void) {
	int ret = nf_register_hooks(tester_net_ops, ARRAY_SIZE(tester_net_ops));
	return ret;
}


#if 0
static struct sk_buff * tester_create_msg(u32 size, u32 index) {
	struct sk_buff * skb;
	struct SMsgData* sicad;
	skb = dev_alloc_skb(size + 32);
        if (!skb) {
		printk(KERN_CRIT "skb alloc failed\n");
		return NULL;
	}
	sicad = (struct SMsgData*)skb_put(skb,size);
	sicad->header.target = dsp_id << 16;
	sicad->header.sender = dsp_id << 16;
	sicad->userData = index;
	return skb;
}

static void tester_send_messages(void)
{
	u32 i;
	struct sk_buff * skb;
	struct timeval tv1;
	struct timeval tv2;
	//struct timezone *tz;

	do_gettimeofday(&tv1);
	
	for (i = 0; i < 10; i++)
	{
		skb = tester_create_msg(3000, i);	
		MsgRtrARMSend_loopback(skb);
	}
	do_gettimeofday(&tv2);
	
	//printk(KERN_CRIT "Total Send Time: %llu ns\n", (timeval_to_ns(&tv2) - timeval_to_ns(&tv1)));
	
}
#endif


static void tester_callback(struct sk_buff * skb) {
	static int i=0;
	static struct timeval tv1;
	struct timeval tv2;
	struct SMsgData* sicad;
	sicad = (struct SMsgData*)skb->data;

	//printk(KERN_CRIT "Got looped-back packet... attempting to send: %u, %u\n", skb->len, skb->data_len);
	tester_sendpacket(skb);
	return;
#if 0
        dev_kfree_skb_any(skb);
	return;
#endif
	if (i == 0) {
		do_gettimeofday(&tv1);
	}

	i++;
	if (sicad->userData == 511) {
		do_gettimeofday(&tv2);
		printk(KERN_CRIT "Total Receive Time: %llu ns, received: %u\n", (timeval_to_ns(&tv2) - timeval_to_ns(&tv1)), i);
	}
		
        dev_kfree_skb_any(skb);
}


static int __init tester_init(void)
{
    tester_global.eth0 = dev_get_by_name(&init_net, "eth0");
    tester_global.eth1 = dev_get_by_name(&init_net, "eth1");
	MsgRtrARMRegisterSysComID(dsp_id, tester_callback);	
	tester_setup_nethooks();
	iperf_test_port_ne = htons(iperf_test_port);
//	tester_get_mac();
//tester_send_messages();
	return 0;
}
module_init(tester_init);

static void __exit tester_exit(void)
{
        nf_unregister_hooks(tester_net_ops, ARRAY_SIZE(tester_net_ops));
        dev_put(tester_global.eth0);
        dev_put(tester_global.eth1);
}
module_exit(tester_exit);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("NSN ARM-DSP DMA test driver");

